@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2017 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -25,6 +25,14 @@
 class CSvgGraph extends CSvg {
 
 	const SVG_GRAPH_X_AXIS_HEIGHT = 20;
+	const SVG_GRAPH_DEFAULT_COLOR = '#b0af07';
+	const SVG_GRAPH_DEFAULT_TRANSPARENCY = 5;
+	const SVG_GRAPH_DEFAULT_POINTSIZE = 1;
+	const SVG_GRAPH_DEFAULT_LINE_WIDTH = 1;
+
+	const SVG_GRAPH_X_AXIS_LABEL_MARGIN = 5;
+	const SVG_GRAPH_Y_AXIS_LEFT_LABEL_MARGIN = 5;
+	const SVG_GRAPH_Y_AXIS_RIGHT_LABEL_MARGIN = 12;
 
 	protected $canvas_height;
 	protected $canvas_width;
@@ -89,11 +97,13 @@ class CSvgGraph extends CSvg {
 	protected $left_y_min = null;
 	protected $left_y_max = null;
 	protected $left_y_units = null;
+	protected $left_y_empty = true;
 
 	protected $right_y_show = false;
 	protected $right_y_min = null;
 	protected $right_y_max = null;
 	protected $right_y_units = null;
+	protected $right_y_empty = true;
 
 	protected $right_y_zero = null;
 	protected $left_y_zero = null;
@@ -149,6 +159,7 @@ class CSvgGraph extends CSvg {
 		$this->grid_color = '#' . $theme['gridcolor'];
 
 		$this
+			->addClass(ZBX_STYLE_SVG_GRAPH)
 			->setTimePeriod($options['time_period']['time_from'], $options['time_period']['time_to'])
 			->setXAxis($options['x_axis'])
 			->setYAxisLeft($options['left_y_axis'])
@@ -212,11 +223,18 @@ class CSvgGraph extends CSvg {
 	 * @return CSvgGraph
 	 */
 	public function addMetrics(array $metrics = []) {
+		$metrics_for_each_axes = [
+			GRAPH_YAXIS_SIDE_LEFT => 0,
+			GRAPH_YAXIS_SIDE_RIGHT => 0
+		];
+
 		foreach ($metrics as $i => $metric) {
 			$min_value = null;
 			$max_value = null;
 
-			if ($metric['points']) {
+			if (array_key_exists('points', $metric)) {
+				$metrics_for_each_axes[$metric['options']['axisy']]++;
+
 				foreach ($metric['points'] as $point) {
 					if ($min_value === null || $min_value > $point['value']) {
 						$min_value = $point['value'];
@@ -247,7 +265,7 @@ class CSvgGraph extends CSvg {
 			}
 
 			$this->metrics[$i] = [
-				'name' => $metric['hosts'][0]['name'].NAME_DELIMITER.$metric['name'],
+				'name' => $metric['name'],
 				'itemid' => $metric['itemid'],
 				'units' => $metric['units'],
 				'host' => $metric['hosts'][0],
@@ -255,6 +273,9 @@ class CSvgGraph extends CSvg {
 			];
 		}
 
+		$this->left_y_empty = ($metrics_for_each_axes[GRAPH_YAXIS_SIDE_LEFT] == 0);
+		$this->right_y_empty = ($metrics_for_each_axes[GRAPH_YAXIS_SIDE_RIGHT] == 0);
+
 		return $this;
 	}
 
@@ -294,7 +315,8 @@ class CSvgGraph extends CSvg {
 			$this->left_y_max = $options['max'];
 		}
 		if ($options['units'] !== null) {
-			$this->left_y_units = $options['units'];
+			$units = trim(preg_replace('/\s+/', ' ', $options['units']));
+			$this->left_y_units = htmlspecialchars($units);
 		}
 
 		return $this;
@@ -321,7 +343,8 @@ class CSvgGraph extends CSvg {
 			$this->right_y_max = $options['max'];
 		}
 		if ($options['units'] !== null) {
-			$this->right_y_units = $options['units'];
+			$units = trim(preg_replace('/\s+/', ' ', $options['units']));
+			$this->right_y_units = htmlspecialchars($units);
 		}
 
 		return $this;
@@ -346,51 +369,39 @@ class CSvgGraph extends CSvg {
 	 * @return array
 	 */
 	public function getTimeGridWithPosition() {
-		$intervals = [
-			1 => ['H:i:s', 'H:i:s'],					// 1 second
-			5 => ['H:i:s', 'H:i:s'],					// 5 seconds
-			10 => ['H:i:s', 'H:i:s'],					// 10 seconds
-			30 => ['H:i:s', 'H:i:s'],					// 30 seconds
-			SEC_PER_MIN => ['H:i:s', 'H:i:s'],			// 1 minute
-			SEC_PER_MIN * 2 => ['H:i','H:i'],			// 2 minutes
-			SEC_PER_MIN * 5 => ['H:i', 'H:i'],			// 5 minutes
-			SEC_PER_MIN * 15 => ['H:i', 'H:i'],			// 15 minutes
-			SEC_PER_MIN * 30 => ['H:i', 'H:i'],			// 30 minutes
-			SEC_PER_HOUR => ['H:i', 'H:i'],				// 1 hours
-			SEC_PER_HOUR * 3 => ['H:i', 'n-d H:i'],		// 3 hours
-			SEC_PER_HOUR * 6 => ['H:i', 'n-d H:i'],		// 6 hours
-			SEC_PER_HOUR * 12 => ['H:i', 'n-d H:i'],	// 12 hours
-			SEC_PER_DAY => ['H:i', 'n-d H:i'],			// 1 day
-			SEC_PER_WEEK => ['n-d', 'n-d'],				// 1 week
-			SEC_PER_WEEK * 2 => ['n-d', 'n-d'],			// 2 weeks
-			SEC_PER_MONTH => ['n-d', 'Y-n-d'],			// 30 days
-			SEC_PER_MONTH * 3 => ['Y-n-d', 'Y-n-d'],	// 90 days
-			SEC_PER_MONTH * 4 => ['Y-n-d', 'Y-n-d'],	// 120 days
-			SEC_PER_MONTH * 6 => ['Y-n-d', 'Y-n-d'],	// 180 days
-			SEC_PER_YEAR => ['Y-n-d', 'Y-n-d'],			// 1 year
-			SEC_PER_YEAR * 2 => ['Y-n-d', 'Y-n-d']		// 2 years
-		];
-
 		$period = $this->time_till - $this->time_from;
-		$step = $period / $this->canvas_width * 100; // Grid cell (100px) in seconds.
-		$same_day = (date('d', $this->time_till) === date('d', $this->time_from));
-		$same_year = (date('y', $this->time_till) === date('y', $this->time_from));
-		$time_fmt = 'Y-n-d';
+		$step = round(bcmul(bcdiv($period, $this->canvas_width), 100)); // Grid cell (100px) in seconds.
 
-		foreach ($intervals as $interval => $format) {
-			if ($interval > $step) {
-				$fmt = $same_day && $same_year ? 0 : 1;
-				$time_fmt = $format[$fmt];
-				break;
-			}
+		/*
+		 * In case if requested time period is so small that it is rounded to zero, we are displaying only two
+		 * milestones on X axis - the start and the end of period.
+		 */
+		if ($step == 0) {
+			return [
+				0 => date('H:i:s', $this->time_from),
+				$this->canvas_width => date('H:i:s', $this->time_till)
+			];
 		}
 
-		$grid_values = [];
 		$start = $this->time_from + $step - $this->time_from % $step;
+		$time_formats = ['Y-n-d', 'n-d', 'n-d H:i','H:i', 'H:i:s'];
 
-		for ($clock = $start; $this->time_till >= $clock; $clock += $step) {
-			$relative_pos = round($this->canvas_width - $this->canvas_width * ($this->time_till - $clock) / $period);
-			$grid_values[$relative_pos] = date($time_fmt, $clock);
+		// Search for most appropriate time format.
+		foreach ($time_formats as $fmt) {
+			$grid_values = [];
+
+			for ($clock = $start; $this->time_till >= $clock; $clock += $step) {
+				$relative_pos = round($this->canvas_width - $this->canvas_width * ($this->time_till - $clock) / $period);
+				$grid_values[$relative_pos] = date($fmt, $clock);
+			}
+
+			/**
+			 * If at least two calculated time-strings are equal, proceed with next format. Do that as long as each date
+			 * is different or there is no more time formats to test.
+			 */
+			if (count(array_flip($grid_values)) == count($grid_values) || $fmt === end($time_formats)) {
+				break;
+			}
 		}
 
 		return $grid_values;
@@ -428,7 +439,6 @@ class CSvgGraph extends CSvg {
 	 */
 	public function draw() {
 		$this->applyMissingDataFunc();
-		$this->calculateYAxis();
 		$this->calculateDimensions();
 		$this->calculatePaths();
 
@@ -437,7 +447,7 @@ class CSvgGraph extends CSvg {
 		if ($this->left_y_show) {
 			$this->drawCanvasLeftYAxis();
 		}
-		if ($this->right_y_show && (!$this->left_y_show || $this->max_value_right !== null)) {
+		if ($this->right_y_show) {
 			$this->drawCanvasRightYAxis();
 		}
 		if ($this->x_show) {
@@ -446,6 +456,7 @@ class CSvgGraph extends CSvg {
 
 		$this->drawMetricsLine();
 		$this->drawMetricsPoint();
+		$this->drawMetricsBar();
 
 		$this->drawProblems();
 
@@ -478,27 +489,6 @@ class CSvgGraph extends CSvg {
 		);
 	}
 
-	/**
-	 * Calculate which Y axis will be shown in graph.
-	 */
-	protected function calculateYAxis() {
-		$metrics_for_each_axes = [
-			GRAPH_YAXIS_SIDE_LEFT => 0,
-			GRAPH_YAXIS_SIDE_RIGHT => 0
-		];
-
-		foreach ($this->metrics as $metric) {
-			$metrics_for_each_axes[$metric['options']['axisy']]++;
-		}
-
-		if ($metrics_for_each_axes[GRAPH_YAXIS_SIDE_LEFT] == 0) {
-			$this->left_y_show = false;
-		}
-		if ($metrics_for_each_axes[GRAPH_YAXIS_SIDE_RIGHT] == 0) {
-			$this->right_y_show = false;
-		}
-	}
-
 	/**
 	 * Calculate canvas size, margins and offsets for graph canvas inside SVG element.
 	 */
@@ -516,6 +506,19 @@ class CSvgGraph extends CSvg {
 		if ($this->left_y_max === null) {
 			$this->left_y_max = $this->max_value_left ? : 1;
 		}
+
+		if (bccomp($this->left_y_min, $this->left_y_max) == 0) {
+			$this->left_y_min -= 0.5;
+			$this->left_y_max += 0.5;
+		}
+		elseif (bccomp($this->left_y_min, $this->left_y_max) == 1) {
+			$this->left_y_max = $this->left_y_min + 1;
+		}
+
+		$grid = $this->getValueGrid($this->left_y_min, $this->left_y_max);
+		$this->left_y_min = $grid[0];
+		$this->left_y_max = end($grid);
+
 		if ($this->left_y_units === null) {
 			$this->left_y_units = '';
 			foreach ($this->metrics as $metric) {
@@ -534,6 +537,18 @@ class CSvgGraph extends CSvg {
 			$this->right_y_max = $this->max_value_right ? : 1;
 		}
 
+		if (bccomp($this->right_y_min, $this->right_y_max) == 0) {
+			$this->right_y_min -= 0.5;
+			$this->right_y_max += 0.5;
+		}
+		elseif (bccomp($this->right_y_min, $this->right_y_max) == 1) {
+			$this->right_y_max = $this->right_y_min + 1;
+		}
+
+		$grid = $this->getValueGrid($this->right_y_min, $this->right_y_max);
+		$this->right_y_min = $grid[0];
+		$this->right_y_max = end($grid);
+
 		if ($this->right_y_units === null) {
 			$this->right_y_units = '';
 			foreach ($this->metrics as $metric) {
@@ -548,7 +563,7 @@ class CSvgGraph extends CSvg {
 		$approx_width = 10;
 
 		if ($this->left_y_show) {
-			$values = $this->getValuesGridWithPosition(GRAPH_YAXIS_SIDE_LEFT);
+			$values = $this->getValuesGridWithPosition(GRAPH_YAXIS_SIDE_LEFT, $this->left_y_empty);
 
 			if ($values) {
 				$offset_left = max($this->offset_left, max(array_map('strlen', $values)) * $approx_width);
@@ -557,10 +572,11 @@ class CSvgGraph extends CSvg {
 		}
 
 		if ($this->right_y_show) {
-			$values = $this->getValuesGridWithPosition(GRAPH_YAXIS_SIDE_RIGHT);
+			$values = $this->getValuesGridWithPosition(GRAPH_YAXIS_SIDE_RIGHT, $this->right_y_empty);
 
 			if ($values) {
 				$offset_right = max($this->offset_right, max(array_map('strlen', $values)) * $approx_width);
+				$offset_right += self::SVG_GRAPH_Y_AXIS_RIGHT_LABEL_MARGIN;
 				$this->offset_right = (int) min($offset_right, $this->max_yaxis_width);
 			}
 		}
@@ -584,8 +600,11 @@ class CSvgGraph extends CSvg {
 	 *
 	 * @return array
 	 */
-	protected function getValuesGridWithPosition($side) {
-		if ($side === GRAPH_YAXIS_SIDE_LEFT) {
+	protected function getValuesGridWithPosition($side, $empty_set = false) {
+		if ($empty_set) {
+			$units = '';
+		}
+		elseif ($side === GRAPH_YAXIS_SIDE_LEFT) {
 			$min_value = $this->left_y_min;
 			$max_value = $this->left_y_max;
 			$units = $this->left_y_units;
@@ -596,17 +615,16 @@ class CSvgGraph extends CSvg {
 			$units = $this->right_y_units;
 		}
 
-		$min_value = (float) $min_value;
-		$max_value = (float) $max_value;
-		$grid = $this->getValueGrid($min_value, $max_value);
+		$grid = $empty_set ? [0, 1] : $this->getValueGrid($min_value, $max_value);
+		$min_value = $grid[0];
+		$max_value = end($grid);
 		$delta = ($max_value != $min_value)
 			? $max_value - $min_value
 			: (count($grid) > 1 ? end($grid) - $grid[0] : 1);
-		$y_rows = (count($grid) - 1) ? : 1;
 		$grid_values = [];
 
 		foreach ($grid as $value) {
-			$relative_pos = $this->canvas_height - $this->canvas_height * ($max_value - $value) / $delta;
+			$relative_pos = $this->canvas_height - intval($this->canvas_height * ($max_value - $value) / $delta);
 
 			if ($relative_pos >= 0 && $relative_pos <= $this->canvas_height) {
 				$grid_values[$relative_pos] = convert_units([
@@ -624,8 +642,9 @@ class CSvgGraph extends CSvg {
 	 * Add Y axis with labels to left side of graph.
 	 */
 	protected function drawCanvasLeftYaxis() {
+		$grid_values = $this->getValuesGridWithPosition(GRAPH_YAXIS_SIDE_LEFT, $this->left_y_empty);
 		$this->addItem(
-			(new CSvgGraphAxis($this->getValuesGridWithPosition(GRAPH_YAXIS_SIDE_LEFT), GRAPH_YAXIS_SIDE_LEFT))
+			(new CSvgGraphAxis($grid_values, GRAPH_YAXIS_SIDE_LEFT))
 				->setSize($this->offset_left, $this->canvas_height)
 				->setPosition($this->canvas_x - $this->offset_left, $this->canvas_y)
 				->setTextColor($this->text_color)
@@ -637,8 +656,15 @@ class CSvgGraph extends CSvg {
 	 * Add Y axis with labels to right side of graph.
 	 */
 	protected function drawCanvasRightYAxis() {
+		$grid_values = $this->getValuesGridWithPosition(GRAPH_YAXIS_SIDE_RIGHT, $this->right_y_empty);
+
+		// Do not draw label at the bottom of right Y axis to avoid label averlapping with horizontal axis arrow.
+		if (array_key_exists(0, $grid_values)) {
+			unset($grid_values[0]);
+		}
+
 		$this->addItem(
-			(new CSvgGraphAxis($this->getValuesGridWithPosition(GRAPH_YAXIS_SIDE_RIGHT), GRAPH_YAXIS_SIDE_RIGHT))
+			(new CSvgGraphAxis($grid_values, GRAPH_YAXIS_SIDE_RIGHT))
 				->setSize($this->offset_right, $this->canvas_height)
 				->setPosition($this->canvas_x + $this->canvas_width, $this->canvas_y)
 				->setTextColor($this->text_color)
@@ -670,11 +696,13 @@ class CSvgGraph extends CSvg {
 	protected function getValueGrid($min, $max) {
 		$res = [];
 
+		// If absolute min/max is equal, calculate grid with 4 rows to make 0 centered. 5 rows otherwise.
+		$grid_rows = (abs($min) == abs($max)) ? 4 : 5;
 		$decimals = strlen(substr(strrchr($max, '.'), 1));
 		$decimals = $decimals > 4 ? 4 : $decimals;
 		$decimals = $decimals < 2 ? 2 : $decimals;
 		for ($base = 10; $base > .01; $base /= 10) {
-			$mul = $max ? 1 / pow($base, floor(log10($max))) : 1;
+			$mul = ($max > 0) ? 1 / pow($base, floor(log10($max))) : 1;
 			$max10 = ceil($mul * $max) / $mul;
 			$min10 = floor($mul * $min) / $mul;
 			$delta = $max10 - $min10;
@@ -682,7 +710,7 @@ class CSvgGraph extends CSvg {
 
 			if ($mul >= 1) {
 				if ($delta) {
-					for($i = 0; $delta >= $i; $i += $delta / 5) {
+					for ($i = 0; $delta >= $i; $i += $delta / $grid_rows) {
 						$res[] = sprintf('%.'.$decimals.'f', $i + $min10);
 					}
 				}
@@ -704,12 +732,12 @@ class CSvgGraph extends CSvg {
 		$value_points = [];
 
 		if ($this->left_y_show) {
-			$value_points = $this->getValuesGridWithPosition(GRAPH_YAXIS_SIDE_LEFT);
+			$value_points = $this->getValuesGridWithPosition(GRAPH_YAXIS_SIDE_LEFT, $this->left_y_empty);
 
 			unset($time_points[0]);
 		}
 		elseif ($this->right_y_show) {
-			$value_points = $this->getValuesGridWithPosition(GRAPH_YAXIS_SIDE_RIGHT);
+			$value_points = $this->getValuesGridWithPosition(GRAPH_YAXIS_SIDE_RIGHT, $this->right_y_empty);
 
 			unset($time_points[$this->canvas_width]);
 		}
@@ -729,6 +757,10 @@ class CSvgGraph extends CSvg {
 	 * Calculate paths for metric elements.
 	 */
 	protected function calculatePaths() {
+		// Metric having very big values of y outside visible area will fail to render.
+		$y_max = pow(2, 16);
+		$y_min = $y_max * -1;
+
 		foreach ($this->metrics as $index => $metric) {
 			if (!array_key_exists($index, $this->points)) {
 				continue;
@@ -743,14 +775,14 @@ class CSvgGraph extends CSvg {
 				$max_value = $this->left_y_max;
 			}
 
-			$time_range = $this->time_till - $this->time_from ? : 1;
-			$value_diff = $max_value - $min_value ? : 1;
+			$time_range = ($this->time_till - $this->time_from) ? : 1;
+			$value_diff = ($max_value - $min_value) ? : 1;
 			$timeshift = $metric['options']['timeshift'];
 			$paths = [];
 
 			$path_num = 0;
 			foreach ($this->points[$index] as $clock => $point) {
-				// If missing data function is SVG_GRAPH_MISSING_DATA_NONE, path should be splitted in multiple svg shapes.
+				// If missing data function is SVG_GRAPH_MISSING_DATA_NONE, path should be split in multiple svg shapes.
 				if ($point === null) {
 					$path_num++;
 					continue;
@@ -762,9 +794,15 @@ class CSvgGraph extends CSvg {
 				 */
 				$in_range = ($max_value >= $point && $min_value <= $point);
 				if ($in_range || $metric['options']['type'] != SVG_GRAPH_TYPE_POINTS) {
-					$x = $this->canvas_x + $this->canvas_width - $this->canvas_width * ($this->time_till - $clock + $timeshift) / $time_range;
+					$x = $this->canvas_x + $this->canvas_width
+						- $this->canvas_width * ($this->time_till - $clock + $timeshift) / $time_range;
 					$y = $this->canvas_y + $this->canvas_height * ($max_value - $point) / $value_diff;
-					$paths[$path_num][] = [$x, $y, convert_units([
+
+					if (!$in_range) {
+						$y = ($point > $max_value) ? max($y_min, $y) : min($y_max, $y);
+					}
+
+					$paths[$path_num][] = [$x, ceil($y), convert_units([
 						'value' => $point,
 						'units' => $metric['units']
 					])];
@@ -784,12 +822,12 @@ class CSvgGraph extends CSvg {
 		foreach ($this->metrics as $index => $metric) {
 			/**
 			 * - Missing data points are calculated only between existing data points;
-			 * - Missing data points are not calculated for SVG_GRAPH_TYPE_POINTS metrics;
+			 * - Missing data points are not calculated for SVG_GRAPH_TYPE_POINTS && SVG_GRAPH_TYPE_BAR metrics;
 			 * - SVG_GRAPH_MISSING_DATA_CONNECTED is default behavior of SVG graphs, so no need to calculate anything
 			 *   here.
 			 */
 			if (array_key_exists($index, $this->points)
-					&& $metric['options']['type'] != SVG_GRAPH_TYPE_POINTS
+					&& !in_array($metric['options']['type'], [SVG_GRAPH_TYPE_POINTS, SVG_GRAPH_TYPE_BAR])
 					&& $metric['options']['missingdatafunc'] != SVG_GRAPH_MISSING_DATA_CONNECTED) {
 				$points = &$this->points[$index];
 				$missing_data_points = $this->getMissingData($points, $metric['options']['missingdatafunc']);
@@ -871,7 +909,9 @@ class CSvgGraph extends CSvg {
 		$y_zero = ($metric['options']['axisy'] == GRAPH_YAXIS_SIDE_RIGHT) ? $this->right_y_zero : $this->left_y_zero;
 
 		foreach ($paths as $path) {
-			$this->addItem(new CSvgGraphArea($path, $metric, $y_zero));
+			if (count($path) > 1) {
+				$this->addItem(new CSvgGraphArea($path, $metric, $y_zero));
+			}
 		}
 	}
 
@@ -882,23 +922,11 @@ class CSvgGraph extends CSvg {
 		foreach ($this->metrics as $index => $metric) {
 			if (array_key_exists($index, $this->paths) && ($metric['options']['type'] == SVG_GRAPH_TYPE_LINE
 					|| $metric['options']['type'] == SVG_GRAPH_TYPE_STAIRCASE)) {
-				$group = (new CSvgGroup())
-					->setAttribute('data-set', $metric['options']['type'] == SVG_GRAPH_TYPE_LINE ? 'line' : 'staircase')
-					->setAttribute('data-metric', CHtml::encode($metric['name']))
-					->setAttribute('data-color', $metric['options']['color']);
-
 				if ($metric['options']['fill'] > 0) {
 					$this->drawMetricArea($metric, $this->paths[$index]);
 				}
 
-				foreach ($this->paths[$index] as $path) {
-					$group->addItem(new CSvgGraphLine($path, $metric));
-				}
-
-				$this->addItem($group->addItem(
-					(new CSvgCircle(-10, -10, $metric['options']['width'] + 4))
-						->addClass(CSvgTag::ZBX_STYLE_GRAPH_HIGHLIGHTED_VALUE)
-				));
+				$this->addItem(new CSvgGraphLineGroup($this->paths[$index], $metric));
 			}
 		}
 	}
@@ -914,6 +942,78 @@ class CSvgGraph extends CSvg {
 		}
 	}
 
+	/**
+	 * Add metric of type bar to graph.
+	 */
+	protected function drawMetricsBar() {
+		$bar_min_width = [
+			GRAPH_YAXIS_SIDE_LEFT => $this->canvas_width * .25,
+			GRAPH_YAXIS_SIDE_RIGHT => $this->canvas_width * .25
+		];
+		$bar_groups_indexes = [];
+		$bar_groups_position = [];
+
+		foreach ($this->paths as $index => $path) {
+			if ($this->metrics[$index]['options']['type'] == SVG_GRAPH_TYPE_BAR) {
+				// If one second in displayed over multiple pixels, this shows number of px in second.
+				$sec_per_px = ceil(($this->time_till - $this->time_from) / $this->canvas_width);
+				$px_per_sec = ceil($this->canvas_width / ($this->time_till - $this->time_from));
+
+				$y_axis_side = $this->metrics[$index]['options']['axisy'];
+				$time_points = array_keys($this->points[$index]);
+				$last_point = 0;
+				$path = reset($path);
+
+				foreach ($path as $point_index => $point) {
+					$time_point = ($sec_per_px > $px_per_sec)
+						? floor($time_points[$point_index] / $sec_per_px) * $sec_per_px
+						: $time_points[$point_index];
+					$bar_groups_indexes[$y_axis_side][$time_point][$index] = $point_index;
+					$bar_groups_position[$y_axis_side][$time_point][$point_index] = $point[0];
+
+					if ($last_point > 0) {
+						$bar_min_width[$y_axis_side] = min($point[0] - $last_point, $bar_min_width[$y_axis_side]);
+					}
+					$last_point = $point[0];
+				}
+			}
+		}
+
+		foreach ($bar_groups_indexes as $y_axis => $points) {
+			foreach ($points as $time_point => $paths) {
+				$group_count = count($paths);
+				$group_width = $bar_min_width[$y_axis];
+				$bar_width = ceil($group_width / $group_count * .75);
+				$group_index = 0;
+				foreach ($paths as $path_index => $point_index) {
+					$group_x = $bar_groups_position[$y_axis][$time_point][$point_index];
+					if ($group_count > 1) {
+						$this->paths[$path_index][0][$point_index][0] = $group_x
+							// Calculate the leftmost X-coordinate including gap size.
+							- $group_width * .375
+							// Calculate the X-offset for the each bar in the group.
+							+ ceil($bar_width * ($group_index + .5));
+						$group_index++;
+					}
+					$this->paths[$path_index][0][$point_index][3] = max(1, $bar_width);
+					// X position for bars group.
+					$this->paths[$path_index][0][$point_index][4] = $group_x - $group_width * .375;
+				}
+			}
+		}
+
+		foreach ($this->metrics as $index => $metric) {
+			if ($metric['options']['type'] == SVG_GRAPH_TYPE_BAR && array_key_exists($index, $this->paths)) {
+				$metric['options']['y_zero'] = ($metric['options']['axisy'] == GRAPH_YAXIS_SIDE_RIGHT)
+					? $this->right_y_zero
+					: $this->left_y_zero;
+				$metric['options']['bar_width'] = $bar_min_width[$metric['options']['axisy']];
+
+				$this->addItem(new CSvgGraphBar(reset($this->paths[$index]), $metric));
+			}
+		}
+	}
+
 	/**
 	 * Add problems tooltip data to graph.
 	 */
@@ -923,10 +1023,14 @@ class CSvgGraph extends CSvg {
 
 		foreach ($this->problems as $problem) {
 			// If problem is never recovered, it will be drown till the end of graph or till current time.
-			$time_to =  ($problem['r_clock'] == 0) ? min($this->time_till, time()) : $problem['r_clock'];
+			$time_to =  ($problem['r_clock'] == 0)
+				? min($this->time_till, time())
+				: min($this->time_till, $problem['r_clock']);
 			$time_range = $this->time_till - $this->time_from;
-			$x1 = $this->canvas_x + $this->canvas_width - $this->canvas_width * ($this->time_till - $problem['clock']) / $time_range;
-			$x2 = $this->canvas_x + $this->canvas_width - $this->canvas_width * ($this->time_till - $time_to) / $time_range;
+			$x1 = $this->canvas_x + $this->canvas_width
+				- $this->canvas_width * ($this->time_till - $problem['clock']) / $time_range;
+			$x2 = $this->canvas_x + $this->canvas_width
+				- $this->canvas_width * ($this->time_till - $time_to) / $time_range;
 
 			if ($this->canvas_x > $x1) {
 				$x1 = $this->canvas_x;
@@ -965,7 +1069,7 @@ class CSvgGraph extends CSvg {
 					->setArgument('eventid', $problem['eventid'])
 					->getUrl(),
 				'r_eventid' => $problem['r_eventid'],
-				'severity' => ($problem['r_clock'] == 0) ? getSeverityStyle($problem['severity']) : '',
+				'severity' => getSeverityStyle($problem['severity'], $problem['r_clock'] == 0),
 				'status' => $status_str,
 				'status_color' => $status_color
 			];
@@ -973,12 +1077,12 @@ class CSvgGraph extends CSvg {
 			// At least 3 pixels expected to be occupied to show the range. Show simple anotation otherwise.
 			$draw_type = ($x2 - $x1) > 2 ? CSvgGraphAnnotation::TYPE_RANGE : CSvgGraphAnnotation::TYPE_SIMPLE;
 
-			// Draw border lines. Make them dashed if beginning or ending of highligted zone is visible in graph.
-			if ($problem['clock'] >= $this->time_from) {
+			// Draw border lines. Make them dashed if beginning or ending of highlighted zone is visible in graph.
+			if ($problem['clock'] > $this->time_from) {
 				$draw_type |= CSvgGraphAnnotation::DASH_LINE_START;
 			}
 
-			if ($this->time_till >= $time_to) {
+			if ($this->time_till > $time_to) {
 				$draw_type |= CSvgGraphAnnotation::DASH_LINE_END;
 			}
 
