@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -45,25 +45,27 @@ class CSvgGraphHelper {
 	public static function get(array $options = [], $width, $height) {
 		$metrics = [];
 		$errors = [];
-		$problems = [];
 
 		// Find which metrics will be shown in graph and calculate time periods and display options.
 		self::getMetrics($metrics, $options['data_sets']);
 		// Apply overrides for previously selected $metrics.
+
+		$metrics = CMacrosResolverHelper::resolveItemNames($metrics);
+		$metrics = CArrayHelper::renameObjectsKeys($metrics, ['name_expanded' => 'name']);
+
 		self::applyOverrides($metrics, $options['overrides']);
 		// Apply time periods for each $metric, based on graph/dashboard time as well as metric level timeshifts.
 		self::getTimePeriods($metrics, $options['time_period']);
 		// Find what data source (history or trends) will be used for each metric.
 		self::getGraphDataSource($metrics, $errors, $options['data_source'], $width);
 		// Load Data for each metric.
-		self::getMetricsData($metrics, $errors, $width);
+		self::getMetricsData($metrics, $width);
+		// Load aggregated Data for each dataset.
+		self::getMetricsAggregatedData($metrics);
 
 		// Legend single line height is 18. Value should be synchronized with $svg-legend-line-height in 'screen.scss'.
 		$legend_height = ($options['legend'] == SVG_GRAPH_LEGEND_TYPE_SHORT) ? $options['legend_lines'] * 18 : 0;
 
-		$metrics = CMacrosResolverHelper::resolveItemNames($metrics);
-		$metrics = CArrayHelper::renameObjectsKeys($metrics, ['name_expanded' => 'name']);
-
 		// Draw SVG graph.
 		$graph = (new CSvgGraph([
 			'time_period' => $options['time_period'],
@@ -90,7 +92,7 @@ class CSvgGraphHelper {
 
 			foreach ($metrics as $metric) {
 				$labels[] = [
-					'name' => $metric['hosts'][0]['name'].NAME_DELIMITER.$metric['name'],
+					'name' => $metric['name'],
 					'color' => $metric['options']['color']
 				];
 			}
@@ -130,20 +132,157 @@ class CSvgGraphHelper {
 		];
 	}
 
+	/**
+	 * Select aggregated data to show in graph for each metric.
+	 */
+	protected static function getMetricsAggregatedData(array &$metrics = []) {
+		$dataset_metrics = [];
+
+		foreach ($metrics as $metric_num => &$metric) {
+			if ($metric['options']['aggregate_function'] == GRAPH_AGGREGATE_NONE) {
+				continue;
+			}
+
+			$dataset_num = $metric['data_set'];
+
+			if ($metric['options']['aggregate_grouping'] == GRAPH_AGGREGATE_BY_ITEM) {
+				$name = $metric['hosts'][0]['name'].NAME_DELIMITER.$metric['name'];
+			}
+			else {
+				$name = 'Dataset #'.($dataset_num + 1);
+			}
+
+			$item = [
+				'itemid' => $metric['itemid'],
+				'value_type' => $metric['value_type'],
+				'source' => ($metric['source'] == SVG_GRAPH_DATA_SOURCE_HISTORY) ? 'history' : 'trends'
+			];
+
+			if (!array_key_exists($dataset_num, $dataset_metrics)) {
+				$metric = array_merge($metric, [
+					'name' => graph_item_aggr_fnc2str($metric['options']['aggregate_function']).'('.$name.')',
+					'items' => [],
+					'points' => []
+				]);
+				$metric['options']['aggregate_interval'] =
+					(int) timeUnitToSeconds($metric['options']['aggregate_interval'], true);
+
+				if ($metric['options']['aggregate_grouping'] == GRAPH_AGGREGATE_BY_DATASET) {
+					$dataset_metrics[$dataset_num] = $metric_num;
+				}
+
+				$metric['items'][] = $item;
+			}
+			else {
+				$metrics[$dataset_metrics[$dataset_num]]['items'][] = $item;
+				unset($metrics[$metric_num]);
+			}
+		}
+		unset($metric);
+
+		foreach ($metrics as &$metric) {
+			if ($metric['options']['aggregate_function'] == GRAPH_AGGREGATE_NONE) {
+				continue;
+			}
+
+			$result = Manager::History()->getGraphAggregationByInterval(
+				$metric['items'], $metric['time_period']['time_from'], $metric['time_period']['time_to'],
+				$metric['options']['aggregate_function'], $metric['options']['aggregate_interval']
+			);
+
+			$metric_points = [];
+
+			if ($result) {
+				foreach ($result as $itemid => $points) {
+					foreach ($points['data'] as $point) {
+						$metric_points[$point['tick']]['itemid'][] = $point['itemid'];
+						$metric_points[$point['tick']]['clock'][] = $point['clock'];
+
+						if (array_key_exists('count', $point)) {
+							$metric_points[$point['tick']]['count'][] = $point['count'];
+						}
+						if (array_key_exists('value', $point)) {
+							$metric_points[$point['tick']]['value'][] = $point['value'];
+						}
+					}
+				}
+				ksort($metric_points, SORT_NUMERIC);
+
+				switch ($metric['options']['aggregate_function']) {
+					case GRAPH_AGGREGATE_MIN:
+						foreach ($metric_points as $tick => $point) {
+							$metric['points'][] = ['clock' => $tick, 'value' => min($point['value'])];
+						}
+						break;
+					case GRAPH_AGGREGATE_MAX:
+						foreach ($metric_points as $tick => $point) {
+							$metric['points'][] = ['clock' => $tick, 'value' => max($point['value'])];
+						}
+						break;
+					case GRAPH_AGGREGATE_AVG:
+						foreach ($metric_points as $tick => $point) {
+							$metric['points'][] = [
+								'clock' => $tick,
+								'value' => array_sum($point['value']) / count($point['value'])
+							];
+						}
+						break;
+					case GRAPH_AGGREGATE_COUNT:
+						foreach ($metric_points as $tick => $point) {
+							$metric['points'][] = ['clock' => $tick, 'value' => array_sum($point['count'])];
+						}
+						break;
+					case GRAPH_AGGREGATE_SUM:
+						foreach ($metric_points as $tick => $point) {
+							$metric['points'][] = ['clock' => $tick, 'value' => array_sum($point['value'])];
+						}
+						break;
+					case GRAPH_AGGREGATE_FIRST:
+						foreach ($metric_points as $tick => $point) {
+							if ($point['clock']) {
+								$metric['points'][] = [
+									'clock' => $tick,
+									'value' => $point['value'][array_search(min($point['clock']), $point['clock'])]
+								];
+							}
+						}
+						break;
+					case GRAPH_AGGREGATE_LAST:
+						foreach ($metric_points as $tick => $point) {
+							if ($point['clock']) {
+								$metric['points'][] = [
+									'clock' => $tick,
+									'value' => $point['value'][array_search(max($point['clock']), $point['clock'])]
+								];
+							}
+						}
+						break;
+				}
+			}
+		}
+	}
+
 	/**
 	 * Select data to show in graph for each metric.
 	 */
-	protected static function getMetricsData(array &$metrics = [], array &$errors = [], $width) {
+	protected static function getMetricsData(array &$metrics = [], $width) {
 		// To reduce number of requests, group metrics by time range.
 		$tr_groups = [];
 		foreach ($metrics as $metric_num => &$metric) {
+			if ($metric['options']['aggregate_function'] != GRAPH_AGGREGATE_NONE) {
+				continue;
+			}
+
+			$metric['name'] = $metric['hosts'][0]['name'].NAME_DELIMITER.$metric['name'];
 			$metric['points'] = [];
 
 			$key = $metric['time_period']['time_from'].$metric['time_period']['time_to'];
 			if (!array_key_exists($key, $tr_groups)) {
-				$tr_groups[$key]['time'] = [
-					'from' => $metric['time_period']['time_from'],
-					'to' => $metric['time_period']['time_to']
+				$tr_groups[$key] = [
+					'time' => [
+						'from' => $metric['time_period']['time_from'],
+						'to' => $metric['time_period']['time_to']
+					]
 				];
 			}
 
@@ -157,7 +296,7 @@ class CSvgGraphHelper {
 
 		// Request data.
 		foreach ($tr_groups as $tr_group) {
-			$results = Manager::History()->getGraphAggregation($tr_group['items'], $tr_group['time']['from'],
+			$results = Manager::History()->getGraphAggregationByWidth($tr_group['items'], $tr_group['time']['from'],
 				$tr_group['time']['to'], $width
 			);
 
@@ -180,8 +319,6 @@ class CSvgGraphHelper {
 				unset($metric);
 			}
 		}
-
-		return $metrics;
 	}
 
 	/**
@@ -317,21 +454,19 @@ class CSvgGraphHelper {
 			}
 		}
 
-		// Filter out only items of selected metrics.
-		if ($problem_options['itemids'] !== null) {
-			$options['objectids'] = array_keys(API::Trigger()->get([
-				'output' => [],
-				'hostids' => array_key_exists('hostids', $options) ? $options['hostids'] : null,
-				'itemids' => $problem_options['itemids'],
-				'preservekeys' => true
-			]));
+		$options['objectids'] = array_keys(API::Trigger()->get([
+			'output' => [],
+			'hostids' => array_key_exists('hostids', $options) ? $options['hostids'] : null,
+			'itemids' => $problem_options['itemids'],
+			'monitored' => true,
+			'preservekeys' => true
+		]));
 
-			// Return if no triggers found.
-			if (!$options['objectids']) {
-				return [];
-			}
+		unset($options['hostids']);
 
-			unset($options['hostids']);
+		// Return if no triggers found.
+		if (!$options['objectids']) {
+			return [];
 		}
 
 		// Add severity filter.
@@ -384,7 +519,7 @@ class CSvgGraphHelper {
 	protected static function getMetrics(array &$metrics, array $data_sets) {
 		$max_metrics = SVG_GRAPH_MAX_NUMBER_OF_METRICS;
 
-		foreach ($data_sets as $data_set) {
+		foreach ($data_sets as $index => $data_set) {
 			if (!$data_set['hosts'] || !$data_set['items']) {
 				continue;
 			}
@@ -431,7 +566,7 @@ class CSvgGraphHelper {
 
 				unset($data_set['hosts'], $data_set['items']);
 
-				// The bigger transparency level, the less visibile the metric is.
+				// The bigger transparency level, the less visible the metric is.
 				$data_set['transparency'] = 10 - (int) $data_set['transparency'];
 
 				$data_set['timeshift'] = ($data_set['timeshift'] !== '')
@@ -442,7 +577,7 @@ class CSvgGraphHelper {
 
 				foreach ($items as $item) {
 					$data_set['color'] = array_shift($colors);
-					$metrics[] = $item + ['options' => $data_set];
+					$metrics[] = $item + ['data_set' => $index, 'options' => $data_set];
 					$max_metrics--;
 				}
 			}
@@ -515,7 +650,7 @@ class CSvgGraphHelper {
 					: null;
 
 				if (array_key_exists('transparency', $override)) {
-					// The bigger transparency level, the less visibile the metric is.
+					// The bigger transparency level, the less visible the metric is.
 					$override['transparency'] = 10 - (int) $override['transparency'];
 				}
 
@@ -574,27 +709,13 @@ class CSvgGraphHelper {
 	/**
 	 * Prepare an array to be used for hosts/items filtering.
 	 *
-	 * @param string   $patterns  String containing hosts/items patterns.
+	 * @param array  $patterns  Array of strings containing hosts/items patterns.
 	 *
-	 * @return mixed|array  Returns array of patterns or NULL if all tested hosts/items are valid.
+	 * @return array|mixed  Returns array of patterns.
+	 *                      Returns NULL if array contains '*' (so any possible host/item search matches).
 	 */
-	protected static function processPattern($patterns) {
-		$patterns = array_keys(array_flip(preg_split('/(\r\n|\n|,)/', $patterns)));
-
-		foreach ($patterns as &$pattern) {
-			$pattern = trim($pattern);
-			if ($pattern === '*') {
-				$patterns = null;
-				break;
-			}
-		}
-		unset($pattern);
-
-		if ($patterns !== null) {
-			$patterns = array_filter($patterns, 'strlen');
-		}
-
-		return $patterns;
+	protected static function processPattern(array $patterns) {
+		return in_array('*', $patterns) ? null : $patterns;
 	}
 
 	/*
