@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -44,7 +44,7 @@ jQuery.noConflict();
 	};
 }(jQuery));
 
-var overlays_stack = [];
+var overlays_stack = new OverlayCollection();
 
 function isset(key, obj) {
 	return (is_null(key) || is_null(obj)) ? false : (typeof(obj[key]) != 'undefined');
@@ -110,6 +110,21 @@ function is_array(obj) {
 	return (obj != null) && (typeof obj == 'object') && ('splice' in obj) && ('join' in obj);
 }
 
+/**
+ * Get elements existing exclusively in one of both arrays.
+ *
+ * @param {Array} arr
+ *
+ * @returns {Array}
+ */
+Array.prototype.xor = function(arr) {
+	var merged_arr = this.concat(arr);
+
+	return merged_arr.filter(function(e) {
+		return (merged_arr.indexOf(e) === merged_arr.lastIndexOf(e));
+	});
+};
+
 function addListener(element, eventname, expression, bubbling) {
 	bubbling = bubbling || false;
 	element = $(element);
@@ -160,71 +175,17 @@ function cancelEvent(e) {
 	return false;
 }
 
-function add_variable(o_el, s_name, x_value, s_formname, o_document) {
-	var form;
-
-	if (!o_document) {
-		o_document = document;
-	}
-
-	if (s_formname) {
-		if (!(form = o_document.forms[s_formname])) {
-			throw "Missing form with name '" + s_formname + "'.";
-		}
-	}
-	else if (o_el) {
-		if (!(form = o_el.form)) {
-			throw "Missing form in 'o_el' object";
-		}
-	}
-	else {
-		if (!(form = this.form)) {
-			throw "Missing form in 'this' object";
-		}
-	}
-
-	var o_variable = o_document.createElement('input');
-	if (!o_variable) {
-		throw "Can't create element";
-	}
-	o_variable.type = 'hidden';
-	o_variable.name = s_name;
-	o_variable.id = s_name;
-	o_variable.value = x_value;
-	form.appendChild(o_variable);
-
-	return true;
-}
-
 function checkAll(form_name, chkMain, shkName) {
 	var frmForm = document.forms[form_name],
 		value = frmForm.elements[chkMain].checked;
 
 	chkbxRange.checkObjectAll(shkName, value);
 	chkbxRange.update(shkName);
-	chkbxRange.saveCookies(shkName);
+	chkbxRange.saveSessionStorage(shkName);
 
 	return true;
 }
 
-function checkLocalAll(form_name, chkMain, chkName) {
-	var frmForm = document.forms[form_name];
-	var checkboxes = $$('input[name=' + chkName + ']');
-
-	for (var i = 0; i < checkboxes.length; i++) {
-		if (isset('type', checkboxes[i]) && checkboxes[i].type == 'checkbox') {
-			checkboxes[i].checked = frmForm.elements[chkMain].checked;
-		}
-	}
-
-	return true;
-}
-
-function close_window() {
-	window.setTimeout('window.close();', 500); // solve bug for Internet Explorer
-	return false;
-}
-
 function Confirm(msg) {
 	return confirm(msg);
 }
@@ -367,60 +328,6 @@ function get_bodywidth() {
 	return (w2 < w) ? w2 : w;
 }
 
-function get_cursor_position(e) {
-	e = e || window.event;
-	var cursor = {x: 0, y: 0};
-
-	if (e.pageX || e.pageY) {
-		cursor.x = e.pageX;
-		cursor.y = e.pageY;
-	}
-	else {
-		var de = document.documentElement;
-		var b = document.body;
-		cursor.x = e.clientX + (de.scrollLeft || b.scrollLeft) - (de.clientLeft || 0);
-		cursor.y = e.clientY + (de.scrollTop || b.scrollTop) - (de.clientTop || 0);
-	}
-
-	return cursor;
-}
-
-function get_scroll_pos() {
-	var scrOfX = 0, scrOfY = 0;
-
-	// netscape compliant
-	if (typeof(window.pageYOffset) == 'number') {
-		scrOfY = window.pageYOffset;
-		scrOfX = window.pageXOffset;
-	}
-	// DOM compliant
-	else if (document.body && (document.body.scrollLeft || document.body.scrollTop)) {
-		scrOfY = document.body.scrollTop;
-		scrOfX = document.body.scrollLeft;
-	}
-	// IE6 standards compliant mode
-	else if (document.documentElement && (document.documentElement.scrollLeft || document.documentElement.scrollTop)) {
-		scrOfY = document.documentElement.scrollTop;
-		scrOfX = document.documentElement.scrollLeft;
-	}
-
-	return [scrOfX, scrOfY];
-}
-
-function openWinCentered(url, name, width, height, params) {
-	var top = Math.ceil((screen.height - height) / 2),
-		left = Math.ceil((screen.width - width) / 2);
-
-	if (params.length > 0) {
-		params = ', ' + params;
-	}
-
-	var windowObj = window.open(new Curl(url).getUrl(), name,
-		'width=' + width + ', height=' + height + ', top=' + top + ', left=' + left + params
-	);
-	windowObj.focus();
-}
-
 /**
  * Opens popup content in overlay dialogue.
  *
@@ -466,7 +373,8 @@ function PopUp(action, options, dialogueid, trigger_elmnt) {
 				buttons.push({
 					'title': t('Cancel'),
 					'class': 'btn-alt',
-					'action': function() {}
+					'cancel': true,
+					'action': (typeof resp.cancel_action !== 'undefined') ? resp.cancel_action : function() {}
 				});
 
 				ovelay_properties['title'] = resp.header;
@@ -493,40 +401,21 @@ function PopUp(action, options, dialogueid, trigger_elmnt) {
 /**
  * Function to add details about overlay UI elements in global overlays_stack variable.
  *
- * @param {string} dialogueid	Unique overlay element identifier.
- * @param {object} element		UI element which must be focused when overlay UI element will be closed.
- * @param {object} type			Type of overlay UI element.
- * @param {object} xhr			(optional) XHR request used to load content. Used to abort loading. Currently used with
- *								type 'popup' only.
+ * @param {string} id       Unique overlay element identifier.
+ * @param {object} element  UI element which must be focused when overlay UI element will be closed.
+ * @param {object} type     Type of overlay UI element.
+ * @param {object} xhr      (optional) XHR request used to load content. Used to abort loading. Currently used with
+ *                          type 'popup' only.
  */
 function addToOverlaysStack(id, element, type, xhr) {
-	var index = null,
-		id = id.toString();
 
-	jQuery(overlays_stack).each(function(i, item) {
-		if (item.dialogueid === id) {
-			index = i;
-			return;
-		}
+	overlays_stack.pushUnique({
+		dialogueid: id.toString(),
+		element: element,
+		type: type,
+		xhr: xhr
 	});
 
-	if (index === null) {
-		// Add new overlay.
-		overlays_stack.push({
-			dialogueid: id,
-			element: element,
-			type: type,
-			xhr: xhr
-		});
-	}
-	else {
-		overlays_stack[index]['element'] = element;
-
-		// Move existing overlay to the end of array.
-		overlays_stack.push(overlays_stack[index]);
-		overlays_stack.splice(index, 1);
-	}
-
 	// Only one instance of handler should be present at any time.
 	jQuery(document)
 		.off('keydown', closeDialogHandler)
@@ -536,12 +425,12 @@ function addToOverlaysStack(id, element, type, xhr) {
 // Keydown handler. Closes last opened overlay UI element.
 function closeDialogHandler(event) {
 	if (event.which == 27) { // ESC
-		var dialog = overlays_stack[overlays_stack.length - 1];
+		var dialog = overlays_stack.end();
 		if (typeof dialog !== 'undefined') {
 			switch (dialog.type) {
 				// Close overlay popup.
 				case 'popup':
-					overlayDialogueDestroy(dialog.dialogueid, dialog.xhr);
+					overlayDialogueDestroy(dialog.dialogueid);
 					break;
 
 				// Close overlay hintbox.
@@ -549,9 +438,14 @@ function closeDialogHandler(event) {
 					hintBox.hideHint(dialog.element, true);
 					break;
 
-				// Close context menu overlays.
-				case 'contextmenu':
-					jQuery('.action-menu.action-menu-top:visible').menuPopup('close', dialog.element);
+				// Close popup menu overlays.
+				case 'menu-popup':
+					jQuery('.menu-popup.menu-popup-top:visible').menuPopup('close', dialog.element);
+					break;
+
+				// Close context menu preloader.
+				case 'preloader':
+					overlayPreloaderDestroy(dialog.dialogueid);
 					break;
 
 				// Close overlay time picker.
@@ -575,11 +469,13 @@ function closeDialogHandler(event) {
 	}
 }
 
-/*
+/**
  * Removed overlay from overlays stack and sets focus to source element.
  *
  * @param {string} dialogueid		Id of dialogue, that is being closed.
  * @param {boolean} return_focus	If not FALSE, the element stored in overlay.element will be focused.
+ *
+ * @return {object|undefined|null}  Overlay object, if found.
  */
 function removeFromOverlaysStack(dialogueid, return_focus) {
 	var overlay = null,
@@ -589,28 +485,17 @@ function removeFromOverlaysStack(dialogueid, return_focus) {
 		return_focus = true;
 	}
 
-	jQuery(overlays_stack).each(function(i, item) {
-		if (item.dialogueid === dialogueid) {
-			overlay = item,
-			index = i;
-			return;
-		}
-	});
-
-	if (overlay) {
-		// Focus UI element that was clicked to open an overlay.
-		if (return_focus) {
-			jQuery(overlay.element).focus();
-		}
-
-		// Remove dialogue from the stack.
-		overlays_stack.splice(index, 1);
+	overlay = overlays_stack.removeById(dialogueid);
+	if (overlay && return_focus) {
+		jQuery(overlay.element).focus();
 	}
 
 	// Remove event listener.
 	if (overlays_stack.length == 0) {
 		jQuery(document).off('keydown', closeDialogHandler);
 	}
+
+	return overlay;
 }
 
 /**
@@ -810,13 +695,12 @@ function validate_trigger_expression(formname, dialogueid) {
 }
 
 function redirect(uri, method, needle, invert_needle, add_sid) {
-	if (typeof add_sid === 'undefined') {
-		add_sid = true;
-	}
-	method = method || 'get';
+	method = (method || 'get').toLowerCase();
+	add_sid = (method !== 'get' && (typeof add_sid === 'undefined' || add_sid));
+
 	var url = new Curl(uri, add_sid);
 
-	if (method.toLowerCase() == 'get') {
+	if (method == 'get') {
 		window.location = url.getUrl();
 	}
 	else {
