@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -192,7 +192,8 @@ if ($hosts) {
 	$items = API::Item()->get([
 		'hostids' => array_keys($hosts),
 		'output' => ['itemid', 'name', 'type', 'value_type', 'units', 'hostid', 'state', 'valuemapid', 'status',
-			'error', 'trends', 'history', 'delay', 'key_', 'flags'],
+			'error', 'trends', 'history', 'delay', 'key_', 'flags', 'description'
+		],
 		'selectApplications' => ['applicationid'],
 		'selectItemDiscovery' => ['ts_delete'],
 		'applicationids' => ($applications !== null) ? zbx_objectValues($applications, 'applicationid') : null,
@@ -511,7 +512,7 @@ foreach ($items as $key => $item) {
 	}
 
 	// change
-	$digits = ($item['value_type'] == ITEM_VALUE_TYPE_FLOAT) ? 2 : 0;
+	$digits = ($item['value_type'] == ITEM_VALUE_TYPE_FLOAT) ? ZBX_UNITS_ROUNDOFF_MIDDLE_LIMIT : 0;
 	if ($lastHistory && $prevHistory
 			&& ($item['value_type'] == ITEM_VALUE_TYPE_FLOAT || $item['value_type'] == ITEM_VALUE_TYPE_UINT64)
 			&& (bcsub($lastHistory['value'], $prevHistory['value'], $digits) != 0)) {
@@ -546,6 +547,11 @@ foreach ($items as $key => $item) {
 		$checkbox->setEnabled(false);
 	}
 
+	$item_name = (new CDiv([
+		(new CSpan($item['name_expanded']))->addClass('label'),
+		($item['description'] !== '') ? makeDescriptionIcon($item['description']) : null
+	]))->addClass('action-container');
+
 	$state_css = ($item['state'] == ITEM_STATE_NOTSUPPORTED) ? ZBX_STYLE_GREY : null;
 
 	if ($filter['showDetails']) {
@@ -565,7 +571,7 @@ foreach ($items as $key => $item) {
 			'',
 			$checkbox,
 			'',
-			(new CCol([$item['name_expanded'], BR(), $itemKey]))->addClass($state_css),
+			(new CCol([$item_name, $itemKey]))->addClass($state_css),
 			(new CCol($item['delay']))->addClass($state_css),
 			(new CCol($item['history']))->addClass($state_css),
 			(new CCol($item['trends']))->addClass($state_css),
@@ -582,7 +588,7 @@ foreach ($items as $key => $item) {
 			'',
 			$checkbox,
 			'',
-			(new CCol($item['name_expanded']))->addClass($state_css),
+			(new CCol($item_name))->addClass($state_css),
 			(new CCol($lastClock))->addClass($state_css),
 			(new CCol($lastValue))->addClass($state_css),
 			(new CCol($change))->addClass($state_css),
@@ -664,7 +670,7 @@ foreach ($items as $item) {
 	}
 
 	// column "change"
-	$digits = ($item['value_type'] == ITEM_VALUE_TYPE_FLOAT) ? 2 : 0;
+	$digits = ($item['value_type'] == ITEM_VALUE_TYPE_FLOAT) ? ZBX_UNITS_ROUNDOFF_MIDDLE_LIMIT : 0;
 	if (isset($lastHistory['value']) && isset($prevHistory['value'])
 			&& ($item['value_type'] == ITEM_VALUE_TYPE_FLOAT || $item['value_type'] == ITEM_VALUE_TYPE_UINT64)
 			&& (bcsub($lastHistory['value'], $prevHistory['value'], $digits) != 0)) {
@@ -698,6 +704,11 @@ foreach ($items as $item) {
 		$checkbox->setEnabled(false);
 	}
 
+	$item_name = (new CDiv([
+		(new CSpan($item['name_expanded']))->addClass('label'),
+		($item['description'] !== '') ? makeDescriptionIcon($item['description']) : null
+	]))->addClass('action-container');
+
 	$state_css = ($item['state'] == ITEM_STATE_NOTSUPPORTED) ? ZBX_STYLE_GREY : null;
 
 	$host = $hosts[$item['hostid']];
@@ -718,7 +729,7 @@ foreach ($items as $item) {
 			'',
 			$checkbox,
 			'',
-			(new CCol([$item['name_expanded'], BR(), $itemKey]))->addClass($state_css),
+			(new CCol([$item_name, $itemKey]))->addClass($state_css),
 			(new CCol($item['delay']))->addClass($state_css),
 			(new CCol($item['history']))->addClass($state_css),
 			(new CCol($item['trends']))->addClass($state_css),
@@ -735,7 +746,7 @@ foreach ($items as $item) {
 			'',
 			$checkbox,
 			'',
-			(new CCol($item['name_expanded']))->addClass($state_css),
+			(new CCol($item_name))->addClass($state_css),
 			(new CCol($lastClock))->addClass($state_css),
 			(new CCol($lastValue))->addClass($state_css),
 			(new CCol($change))->addClass($state_css),
