@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -220,6 +220,15 @@ elseif ($config == QUEUE_DETAILS) {
 		'preservekeys' => true
 	]);
 
+	if (count($queueData) != count($items)) {
+		$items += API::DiscoveryRule()->get([
+			'output' => ['itemid', 'hostid', 'name', 'key_'],
+			'selectHosts' => ['name'],
+			'itemids' => array_diff(array_keys($queueData), array_keys($items)),
+			'preservekeys' => true
+		]);
+	}
+
 	$items = CMacrosResolverHelper::resolveItemNames($items);
 
 	// get hosts for queue items
@@ -240,6 +249,8 @@ elseif ($config == QUEUE_DETAILS) {
 		}
 	}
 
+	$proxies = [];
+
 	if ($proxyHostIds) {
 		$proxies = API::Proxy()->get([
 			'proxyids' => $proxyHostIds,
@@ -252,7 +263,8 @@ elseif ($config == QUEUE_DETAILS) {
 		_('Scheduled check'),
 		_('Delayed by'),
 		_('Host'),
-		_('Name')
+		_('Name'),
+		_('Proxy')
 	]);
 
 	$i = 0;
@@ -273,10 +285,11 @@ elseif ($config == QUEUE_DETAILS) {
 		$table->addRow([
 			zbx_date2str(DATE_TIME_FORMAT_SECONDS, $itemData['nextcheck']),
 			zbx_date2age($itemData['nextcheck']),
-			isset($proxies[$hosts[$item['hostid']]['proxy_hostid']])
-				? $proxies[$hosts[$item['hostid']]['proxy_hostid']]['host'].NAME_DELIMITER.$host['name']
-				: $host['name'],
-			$item['name_expanded']
+			$host['name'],
+			$item['name_expanded'],
+			array_key_exists($hosts[$item['hostid']]['proxy_hostid'], $proxies)
+				? $proxies[$hosts[$item['hostid']]['proxy_hostid']]['host']
+				: ''
 		]);
 	}
 }
